%% HOUSING BOOM-BUST CYCLES AND ASYMMETRIC MACROPRUDENTIAL POLICY
%
% Runs models with 1 and 2 OBCs, retreives and plots policy functions
% William Gatt


clear all; clc;
warning off;
set(groot,'defaultlinelinewidth',1);
set(groot,'defaultAxesXGrid','off');
set(groot,'defaultAxesYGrid','off');
set(groot, 'defaultAxesTickLabelInterpreter','latex')
set(groot, 'defaultLegendInterpreter','latex');
set(groot, 'defaultTextInterpreter','latex');


save_dir = '../Figures';

addpath functions
addpath functions/toolkit_files/
my_colours1;


    
%% SIMULATION
% ---------------------------------------------------------------------
% Sequence of values for state variable j
% ---------------------------------------------------------------------
%   note: we need to shock j, by setting the value of the shock that 
%   feeds into
%   log(j_t) = p.log(j_t-1) + (1-p).log(j_ss) + eps
%   Make sure a value in eps_vec is centered around 0 to make it easier 
%   to plot and make sense of the kink
% ---------------------------------------------------------------------
    
warning off    
    
j_ss  = 0.06;
    
    eps_vec = [[-0.5:0.1:0] [0.025:0.025:1.5]]'; % 0.5 to 1.5 times j_ss (negative domain is linear, no need for fine grid!)
        
    RUN_SIMULATIONS = 0;  % =1 draw policy functions over grid off j (otherwise load results if they exist)


    if RUN_SIMULATIONS 
        
        tic;
        run_borrconst_DR;
        save policy_functions_symMP.mat

        clearvars -except j_ss eps_vec printtt directory tic;

        run_borrconst_asy_DR;       
        eltime=toc;
        save policy_functions_asyMP.mat

        clearvars -except j_ss eps_vec printtt directory tic eltime;

        
        fprintf('\n\nTotal simulation time: %4.2f seconds\n', eltime);
    end
    
    
% Loading all policy functions across different models
load policy_functions_symMP.mat
nn = 1;
    PF_Cb(:,nn) =  Cb_PF;
    PF_Cs(:,nn) =  Cs_PF;
    PF_Hb(:,nn) =  Hb_PF;
    PF_Hs(:,nn) =  Hs_PF;
    PF_q(:,nn)  =  q_PF;
    PF_Y(:,nn)  =  Y_PF;
    PF_B4Y(:,nn)=  B_4Y_PF;
    PF_B(:,nn)  =  B_PF;
    PF_r4(:,nn) =  r4_PF;
    PF_m(:,nn)  =  m_PF;
    PF_PI(:,nn) =  PI_PF;
    PF_mju(:,nn)=  mju_state;
    PF_j(:,nn)  =  j_state;


load policy_functions_asyMP.mat
nn = 2;
    PF_Cb(:,nn) =  Cb_PF;
    PF_Cs(:,nn) =  Cs_PF;
    PF_Hb(:,nn) =  Hb_PF;
    PF_Hs(:,nn) =  Hs_PF;
    PF_q(:,nn)  =  q_PF;
    PF_Y(:,nn)  =  Y_PF;
    PF_B4Y(:,nn)=  B_4Y_PF;
    PF_B(:,nn)  =  B_PF;
    PF_r4(:,nn) =  r4_PF;
    PF_m(:,nn)  =  m_PF;
    PF_PI(:,nn) =  PI_PF;
    PF_mju(:,nn)=  mju_state;
    PF_j(:,nn)  =  j_state;
    
disp('Plotting policy functions (decision rules)');
warning off


%% PLOT POLICY FUNCTIONS

    figure('Name','Policy functions - j',...
            'PaperPositionMode','auto',...
            'PaperOrientation','landscape',...
            'PaperSize',[28 20],'Color','White') 
%         'PaperSize',[9 7],'Color','White') 

    statevar2   = (PF_j/j_ss-1);    
    zeroline    = zeros(length(PF_j(:,1)),1);
    
    kink1=find(PF_mju(:,1)<0.0001,1,'first'); % marker at mju = 0
    kink2=find(PF_mju(:,2)<0.0001,1,'first'); % marker at mju = 0
%     kink3=find(statevar2(:,1)>0.9999,1,'first'); 
    kink3=find(statevar2(:,1)==0,1,'first');  % marker at j=j_ss
        
%     xstring = '$j/\overline{j}$';
    xstring = '$\widehat{\mathsf{j}}$';
%     xlimits=[min(statevar2(:,1)) max(statevar2(:,1))]; 
    xlimits=[min(statevar2(:,1)) 1]; 
    
    
    lw = 2;
    
    subplot(2,2,1); grid off;  box on; hold on;  
    hh1=plot(statevar2(:,1), (PF_Y(:,1)/Y_ss-1)*100,'lineStyle','-','Color',red,'marker','none','markersize',3,'linewidth',lw);
    hh2=plot(statevar2(:,2), (PF_Y(:,2)/Y_ss-1)*100,'lineStyle','-.','Color',lightblue,'marker','none','markersize',3.5,'linewidth',lw);
    hh3=plot(statevar2(:,2), zeroline,'lineStyle','-','linewidth',0.75,'color',grey);
    
    % Markers at mju = 0
    hh4=plot(statevar2(kink1,1), (PF_Y(kink1,1)/Y_ss-1)*100,'lineStyle','none',...
                                                'marker','d','linewidth',2,'MarkerSize',8,...
                                                'MarkerEdgeColor','k','MarkerFaceColor','k');
    plot(statevar2(kink2,1), (PF_Y(kink2,2)/Y_ss-1)*100,'lineStyle','none',...
                                                'marker','d','linewidth',2,'MarkerSize',8,...
                                                'MarkerEdgeColor','k','MarkerFaceColor','k');

    % Marker at j=j_ss
    hh5=plot(statevar2(kink3,1), (PF_Y(kink3,2)/Y_ss-1)*100,'lineStyle','none','marker','o','linewidth',2,'MarkerSize',8,'MarkerEdgeColor','k','MarkerFaceColor','k');
    xlabel(xstring,'interpreter','latex','fontsize',12); 
    ylabel('\% dev. from SS');
    xlim(xlimits);
    xticks(min(xlimits):0.5:max(xlimits));
    ylim([-0.5 0.5]);
    yticks(-0.50:0.25:0.50);
    line([0 0],[min(ylim) max(ylim)],'lineStyle','-','linewidth',0.75,'color',grey);
    set(gca,'FontSize',12)
    title('Output','FontSize', 16);

    
    hL=legend([hh1,hh2,hh5,hh4],{'Symmetric rule','Asymmetric rule','OBC (policy)','OBC (collateral)'},'FontSize',14,'Orientation','horizontal');
            newUnits = 'normalized';
            newPosition = [0.495 -0.015 0.05 0.04];
            

        set(hL,'Position', newPosition,'Units', newUnits,'box', 'off');

    subplot(2,2,2); grid off; box on; hold on;
        hh=plot(statevar2(:,1), (PF_B(:,1)/B_ss-1)*100,'lineStyle','-','Color',red,...
                                            'marker','none','markersize',3,'linewidth',lw);
        hh=plot(statevar2(:,2), (PF_B(:,2)/B_ss-1)*100,'lineStyle','-.','Color',lightblue,...
                                            'marker','none','markersize',3.5,'linewidth',lw);
        hh3=plot(statevar2(:,2), zeroline,'lineStyle','-','linewidth',0.75,'color',grey);

        % Markers at mju = 0
        hh4=plot(statevar2(kink1,1), (PF_B(kink1,1)/B_ss-1)*100,'lineStyle','none',...
                                            'marker','d','linewidth',2,'MarkerSize',8,...
                                            'MarkerEdgeColor','k','MarkerFaceColor','k');
        plot(statevar2(kink2,1), (PF_B(kink2,2)/B_ss-1)*100,'lineStyle','none',...
                                            'marker','d','linewidth',2,'MarkerSize',8,...
                                            'MarkerEdgeColor','k','MarkerFaceColor','k');
        
        % Marker at j=j_ss
        plot(statevar2(kink3,1), (PF_B(kink3,2)/B_ss-1)*100,'lineStyle','none',...
                                            'marker','o','linewidth',2,'MarkerSize',8,...
                                            'MarkerEdgeColor','k','MarkerFaceColor','k');
        xlabel(xstring,'interpreter','latex','fontsize',12); 
        ylabel('\% dev. from SS');
        xlim(xlimits);
        ylim([-15 15]);
        xticks(min(xlimits):0.5:max(xlimits));
                                 
    line([0 0],[min(ylim) max(ylim)],'lineStyle','-','linewidth',0.75,'color',grey);
    set(gca,'FontSize',12)
    title('Credit','FontSize', 16);

    ha = axes('Position',[0 0 1 1],'Xlim',[0 1],'Ylim',[0  1],'Box','off','Visible','off','Units','normalized', 'clipping' , 'off');
%     text(0.45, 0.98,'Policy functions','fontsize',18)

    subplot(2,2,3); grid off;  box on; hold on;
        hh=plot(statevar2(:,1), PF_m(:,1),'lineStyle','-','Color',red,...
                                            'marker','none','markersize',3,'linewidth',lw);
        hh=plot(statevar2(:,2), PF_m(:,2),'lineStyle','-.','Color',lightblue,...
                                            'marker','none','markersize',3.5,'linewidth',lw);
        hh3=plot(statevar2(:,2), zeroline+m_ss,'lineStyle','-','linewidth',0.75,'color',grey);

        % Markers at mju = 0
        plot(statevar2(kink1,1), PF_m(kink1,1),'lineStyle','none',...
                                            'marker','d','linewidth',2,'MarkerSize',8,...
                                            'MarkerEdgeColor','k','MarkerFaceColor','k');
        plot(statevar2(kink2,1), PF_m(kink2,2),'lineStyle','none',...
                                            'marker','d','linewidth',2,'MarkerSize',8,...
                                            'MarkerEdgeColor','k','MarkerFaceColor','k');
        
        % Marker at j=j_ss
        plot(statevar2(kink3,1), PF_m(kink3,2),'lineStyle','none',...
                                            'marker','o','linewidth',2,'MarkerSize',8,...
                                            'MarkerEdgeColor','k','MarkerFaceColor','k');
        xlabel(xstring,'interpreter','latex','fontsize',12); 
        ylabel('Level');
        xlim(xlimits);
        xticks(min(xlimits):0.5:max(xlimits));
        ytickformat('%2.2f')
        line([0 0],[min(ylim) max(ylim)],'lineStyle','-','linewidth',0.75,'color',grey);
        set(gca,'FontSize',12)
        title('LTV ratio','FontSize', 16);        
                                        
        subplot(2,2,4); grid off;  box on; hold on;
        hh1=plot(statevar2(:,1), PF_mju(:,1),'lineStyle','-','Color',red,...
                                'marker','none','markersize',3,'linewidth',lw);
        color1 = get(hh1, 'Color');
        hh2=plot(statevar2(:,2), PF_mju(:,2),'lineStyle','-.','Color',lightblue,...
                                'marker','none','markersize',3.5,'linewidth',lw);
        color2 = get(hh2, 'Color');
        hh3=plot(statevar2(:,2), zeroline+mju_ss,'lineStyle','-','linewidth',0.75,'color',grey);

        
        % Markers at mju = 0
        hh4=plot(statevar2(kink1,1), PF_mju(kink1,1),'lineStyle','none','marker','d',...
                                            'linewidth',2,'MarkerSize',8,'Color',color1,...
                                             'MarkerEdgeColor','k','MarkerFaceColor','k');
        plot(statevar2(kink2,1), PF_mju(kink2,1),'lineStyle','none','marker','d',...
                                            'linewidth',2,'MarkerSize',8,'Color',color2,...
                                            'MarkerEdgeColor','k','MarkerFaceColor','k');
        
        % Marker at j=j_ss
        hh5=plot(statevar2(kink3,1), PF_mju(kink3,1),'lineStyle','none','marker','o',...
                                            'linewidth',2,'MarkerSize',8,'MarkerEdgeColor','k',...
                                            'MarkerFaceColor','k');
        xlabel(xstring,'interpreter','latex','fontsize',12); 
        ylabel('Level');
        ylim([-0.01 0.060]);
        line([0 0],[min(ylim) max(ylim)],'lineStyle','-','linewidth',0.75,'color',grey);
        xlim(xlimits);
        xticks(min(xlimits):0.5:max(xlimits));
        set(gca,'FontSize',12)
        title('Lagrange multiplier','FontSize', 16);

    % Choose whether to save figure as PDF
    printtt     = 0;
    if  printtt==1
        print(gcf,'-dpdf','-fillpage', strcat(save_dir,'/Fig_PolicyFunctions.pdf'));
    end
    

